/**
 * Utilities to accommodate `fp-ts/Ordering`.
 *
 * @since 0.12.0
 */
import type { Ordering } from "fp-ts/Ordering";
/**
 * Alias for the notion of "less than" in `Ordering`.
 *
 * @example
 * import { Ord } from 'fp-ts/number'
 * import { LT } from 'fp-ts-std/Ordering'
 *
 * assert.strictEqual(Ord.compare(0, 1), LT)
 *
 * @category 3 Functions
 * @since 0.12.0
 */
export declare const LT: Ordering;
/**
 * Alias for the notion of "equal to" in `Ordering`.
 *
 * @example
 * import { Ord } from 'fp-ts/number'
 * import { EQ } from 'fp-ts-std/Ordering'
 *
 * assert.strictEqual(Ord.compare(0, 0), EQ)
 *
 * @category 3 Functions
 * @since 0.12.0
 */
export declare const EQ: Ordering;
/**
 * Alias for the notion of "greater than" in `Ordering`.
 *
 * @example
 * import { Ord } from 'fp-ts/number'
 * import { GT } from 'fp-ts-std/Ordering'
 *
 * assert.strictEqual(Ord.compare(1, 0), GT)
 *
 * @category 3 Functions
 * @since 0.12.0
 */
export declare const GT: Ordering;
//# sourceMappingURL=Ordering.d.ts.map